package com.example.fuzzControll.service.impl;

import com.example.fuzzControll.conf.KittyProperties;
import com.example.fuzzControll.exception.FuzzException;
import com.example.fuzzControll.pojo.vo.TestEntity;
import com.example.fuzzControll.service.GenerateMethodService;
import com.example.fuzzControll.tools.CmdTools;
import com.example.fuzzControll.tools.TestTools;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class GenerateMethodServiceImpl implements GenerateMethodService {
    CmdTools cmdTools = new CmdTools();
    @Autowired
    KittyProperties kitty;

    @Override
    public Map<String, List<String>> generation(TestEntity testEntity) throws FuzzException {
        String cmd = parseParameters(testEntity);
        if (cmd.isEmpty()) {
            return null;
        }
        return cmdTools.runProgramCmdAndResult(cmd);
    }

    public String parseParameters(TestEntity testEntity) {
        switch (testEntity.getTestClassName().toLowerCase()) {
            case "foreach":
                return cmd(testEntity, "-f");
            case "repeat":
                return cmd(testEntity, "-r");
            case "oneof":
                return cmd(testEntity, "-o");
            case "switch":
                return cmd(testEntity, "-s");
            case "pad":
                return cmd(testEntity, "-p");
            case "template":
                return cmd(testEntity, "-t");
            case "meta":
                return cmd(testEntity, "-m");
            case "if":
                return cmd(testEntity, "-c");
            case "ifnot":
                return cmd(testEntity, "-e");
            case "trunc"://have error
                return cmd(testEntity, "-u");
            default:
                log.error("未知变异方法！[{}]", testEntity.getTestClassName());
                return null;
        }
    }

    private String cmd(TestEntity testEntity, String cmd) {
        if (!TestTools.paramsLenghtTest(testEntity.getParamJson().length, 5, "generationMethod"))
            return "";
        String target_host = null;
        String target_port = null;
        String s1 = null;
        String s2 = null;
        String s3 = null;
        try {
            target_host = testEntity.getParamJson()[0];
            target_port = testEntity.getParamJson()[1];
            s1 = testEntity.getParamJson()[2];
            s2 = testEntity.getParamJson()[3];
            s3 = testEntity.getParamJson()[4];
        } catch (Exception e) {
            log.error("生成方法参数解析失败！");
        }
        return kitty.getVenvPath() + " " + kitty.getMethodPath() + "generate_method_test.py " + cmd + " " + s1 + " " + s2 + " " + s3 + " --host=" + target_host + " --port=" + target_port;
    }
}
