package com.example.fuzzControll.service.impl;

import com.example.fuzzControll.conf.KittyProperties;
import com.example.fuzzControll.constents.MutationConstent;
import com.example.fuzzControll.exception.FuzzException;
import com.example.fuzzControll.pojo.vo.TestEntity;
import com.example.fuzzControll.service.MutationService;
import com.example.fuzzControll.tools.CmdTools;
import com.example.fuzzControll.tools.TestTools;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

@Service("mutationService")
@Slf4j
class MutationServiceImpl implements MutationService {
    CmdTools cmdTools = new CmdTools();
    @Autowired
    KittyProperties kitty;


    @Override
    public Map<String, List<String>> generation(TestEntity testEntity) throws FuzzException {
        String cmd = parseParameters(testEntity);
        if (cmd.isEmpty()) {
            return null;
        }
        return cmdTools.runProgramCmdAndResult(cmd);
    }

    public String parseParameters(TestEntity testEntity) {
        switch (testEntity.getTestClassName().toLowerCase()) {
            case "bit":
                return variationGranularityCmd(testEntity, 1);
            case "byte":
                return variationGranularityCmd(testEntity, 2);
            case "sqlinjection":
                return distortionLibCmd(testEntity, 2);
            case "commandinjection":
                return distortionLibCmd(testEntity, 1);
            case "outofbuffer":
                return distortionLibCmd(testEntity, 3);
            case "directorytraversal":
                return distortionLibCmd(testEntity, 4);
            case "8-bitinteger":
                return distortionLibCmd(testEntity, 5);
            case "16-bitinteger":
                return distortionLibCmd(testEntity, 6);
            case "32-bitinteger":
                return distortionLibCmd(testEntity, 7);
            case "bitflip"://noresponse
                return distortionLibCmd(testEntity, 8);
            case "twobitflip"://noresponse
                return distortionLibCmd(testEntity, 9);
            case "fourbitflip"://noresponse
                return distortionLibCmd(testEntity, 10);
            case "byteflip"://noresponse
                return distortionLibCmd(testEntity, 11);
            case "wordflip"://noresponse
                return distortionLibCmd(testEntity, 12);
            case "dwordflip"://noresponse
                return distortionLibCmd(testEntity, 13);
            case "blockremove"://noresponse
                return distortionLibCmd(testEntity, 14);
            case "blockduplicate"://noresponse
                return distortionLibCmd(testEntity, 15);
            case "blockset"://noresponse
                return distortionLibCmd(testEntity, 16);
            case "bitflips"://noresponse
                return distortionLibCmd(testEntity, 17);
            case "byteflips":
                return mutationStrategyCmd(testEntity, 1);
            case "interestint8muta":
                return mutationStrategyCmd(testEntity, 2);
            case "interestint16muta":
                return mutationStrategyCmd(testEntity, 3);
            case "interestint32muta":
                return mutationStrategyCmd(testEntity, 4);
            case "onebyterndom":
                return mutationStrategyCmd(testEntity, 5);
            case "mutibytesrandom":
                return mutationStrategyCmd(testEntity, 6);
            case "deleteonebyterandom":
                return mutationStrategyCmd(testEntity, 7);
            case "deletemutibytesrandom":
                return mutationStrategyCmd(testEntity, 8);
            case "shufflebytesrandom":
                return mutationStrategyCmd(testEntity, 9);
            case "swapadjointwobytes":
                return mutationStrategyCmd(testEntity, 10);
            default:
                log.error("未知变异方法！[{}]", testEntity.getTestClassName());
                return null;
        }
    }

    private String distortionLibCmd(TestEntity testEntity, int methodNum) {
        if (!TestTools.paramsLenghtTest(testEntity.getParamJson().length, 2, "distortionLib" + methodNum))
            return "";
        String dst_ip = null;
        String dst_port = null;
        try {
            dst_ip = testEntity.getParamJson()[0];
            dst_port = testEntity.getParamJson()[1];
        } catch (Exception e) {
            log.error("distortionLib [{}] 参数解析失败！", methodNum);
        }
        return kitty.getVenvPath() + " " + kitty.getMutationPath() + MutationConstent.TEST_MUTATED_LIBS + " -g " + methodNum + " -d " + dst_ip + " -p " + dst_port;
    }


    private String variationGranularityCmd(TestEntity testEntity, int methodNum) {
        if (!TestTools.paramsLenghtTest(testEntity.getParamJson().length, 2, "variationGranularity" + methodNum))
            return "";
        String dst_ip = null;
        String dst_port = null;
        try {
            dst_ip = testEntity.getParamJson()[0];
            dst_port = testEntity.getParamJson()[1];
        } catch (Exception e) {
            log.error("variationGranularity [{}] 参数解析失败！", methodNum);
        }
        return kitty.getVenvPath() + " " + kitty.getMutationPath() + MutationConstent.TEST_GRANULARITY_BIT_BYTE + " -g " + methodNum + " -d " + dst_ip + " -p " + dst_port;
    }
    private String mutationStrategyCmd(TestEntity testEntity, int methodNum) {
        if (!TestTools.paramsLenghtTest(testEntity.getParamJson().length, 2, "mutationStrategy" + methodNum))
            return "";
        String dst_ip = null;
        String dst_port = null;
        try {
            dst_ip = testEntity.getParamJson()[0];
            dst_port = testEntity.getParamJson()[1];
        } catch (Exception e) {
            log.error("mutationStrategy [{}] 参数解析失败！", methodNum);
        }
        return kitty.getVenvPath() + " " + kitty.getMutationPath() + MutationConstent.TEST_MUTATION_STRATEGY + " -g " + methodNum + " -d " + dst_ip + " -p " + dst_port;
    }

}